@@ -31,7 +31,7 @@ pub struct MyState;
 
 impl State for MyState {
     fn update(&mut self, context: &mut Context) -> OptionalTransition {
-        let custom_context = context.get_mut::<CustomContext>().unwrap();
+        let mut custom_context = context.borrow_mut::<CustomContext>().unwrap();
         if custom_context.count == 10 {
             Some(Transition::Quit)
         } else {
@@ -41,7 +41,7 @@ impl State for MyState {
     }
 
     fn render(&mut self, context: &mut Context) -> RenderResult {
-        let custom_context = context.get::<CustomContext>().unwrap();
+        let custom_context = context.borrow::<CustomContext>().unwrap();
         info!("{}: {}", custom_context.message, custom_context.count);
     }
 }
