@@ -2,6 +2,7 @@ use log::*;
 use rc_event_queue::LendingIterator;
 use wolf_engine::contexts::EventContext;
 use wolf_engine::event::EventReader;
+use wolf_engine::utils::trust_cell::Ref;
 use wolf_engine::*;
 
 pub fn main() {
@@ -51,9 +52,9 @@ impl ExampleState {
         }
     }
 
-    pub fn get_event_context(context: &Context) -> &EventContext<usize> {
+    pub fn get_event_context(context: &Context) -> Ref<EventContext<usize>> {
         context
-            .get::<EventContext<usize>>()
+            .borrow::<EventContext<usize>>()
             .expect("the context has no EventContext<usize>")
     }
 }
